@@ -36,6 +36,7 @@ import sys
 import copy
 import json
 import datetime
+import socket
 
 from bs4 import BeautifulSoup
 from Nagstamon.Objects import (GenericHost,
@@ -76,7 +77,14 @@ class IcingaWeb2Server(GenericServer):
         # dummy default empty cgi urls - get filled later when server version is known
         self.cgiurl_services = None
         self.cgiurl_hosts = None
-
+        self.cgiurl_monitoring_health = None
+        
+        # https://github.com/HenriWahl/Nagstamon/issues/400
+        # The displayed name for host and service is the Icinga2 "internal" name and not the display_name from host/service configuration
+        # This name is stored in host/service dict under key 'name' but is also used as dict key for dict containing all hosts/services
+        # The "internal" name must still be used to query IcingaWeb2 and is in dict under key 'real_name' since https://github.com/HenriWahl/Nagstamon/issues/192
+        self.use_display_name_host = True
+        self.use_display_name_service = True
 
     def init_HTTP(self):
         """
@@ -96,7 +104,10 @@ class IcingaWeb2Server(GenericServer):
                     form = login.result.find('form')
                     form_inputs = {}
                     for form_input in ('redirect', 'formUID', 'CSRFToken', 'btn_submit'):
-                        form_inputs[form_input] = form.find('input', {'name': form_input})['value']
+                        if not form.find('input', {'name': form_input}) is None:
+                            form_inputs[form_input] = form.find('input', {'name': form_input})['value']
+                        else:
+                            form_inputs[form_input] = ''
                     form_inputs['username'] = self.username
                     form_inputs['password'] = self.password
     
@@ -109,13 +120,15 @@ class IcingaWeb2Server(GenericServer):
             Get status from Icinga Server - only JSON
         """
         # define CGI URLs for hosts and services
-        if self.cgiurl_hosts == self.cgiurl_services == None:
+        if self.cgiurl_hosts == self.cgiurl_services == self.cgiurl_monitoring_health == None:
             # services (unknown, warning or critical?)
             self.cgiurl_services = {'hard': self.monitor_cgi_url + '/monitoring/list/services?service_state>0&service_state<=3&service_state_type=1&addColumns=service_last_check&format=json', \
                                     'soft': self.monitor_cgi_url + '/monitoring/list/services?service_state>0&service_state<=3&service_state_type=0&addColumns=service_last_check&format=json'}
             # hosts (up or down or unreachable)
             self.cgiurl_hosts = {'hard': self.monitor_cgi_url + '/monitoring/list/hosts?host_state>0&host_state<=2&host_state_type=1&addColumns=host_last_check&format=json', \
                                  'soft': self.monitor_cgi_url + '/monitoring/list/hosts?host_state>0&host_state<=2&host_state_type=0&addColumns=host_last_check&format=json'}
+            # monitoring health
+            self.cgiurl_monitoring_health = self.monitor_cgi_url + '/monitoring/health/info?format=json'
 
         # new_hosts dictionary
         self.new_hosts = dict()
@@ -154,6 +167,21 @@ class IcingaWeb2Server(GenericServer):
                 # check if any error occured
                 self.check_for_error(jsonraw, error, status_code)
 
+                # Check if the backend is running
+                # If it isn't running the last values stored in the database are returned/shown
+                # Unfortunately we need to make a extra request for this and only, if monitoring health is possible
+                if self.cgiurl_monitoring_health:
+                    try:
+                        result = self.FetchURL(self.cgiurl_monitoring_health, giveback='raw')
+                        monitoring_health = json.loads(result.result)[0]
+                        if (monitoring_health['is_currently_running'] == '0'):
+                            return Result(result=monitoring_health,
+                                          error='Icinga2 backend not running')
+                    except json.decoder.JSONDecodeError:
+                        # https://github.com/HenriWahl/Nagstamon/issues/619
+                        # Icinga2 monitoring health status query does not seem to work (on older version?)
+                        self.cgiurl_monitoring_health = None
+
                 hosts = json.loads(jsonraw)
 
                 for host in hosts:
@@ -183,14 +211,23 @@ class IcingaWeb2Server(GenericServer):
                         self.new_hosts[host_name].status_information = BeautifulSoup(h['host_output'].replace('\n', ' ').strip(), 'html.parser').text
                         self.new_hosts[host_name].passiveonly = not(int(h['host_active_checks_enabled']))
                         self.new_hosts[host_name].notifications_disabled = not(int(h['host_notifications_enabled']))
-                        self.new_hosts[host_name].flapping = int(h['host_is_flapping'])
-                        self.new_hosts[host_name].acknowledged = int(h['host_acknowledged'])
-                        self.new_hosts[host_name].scheduled_downtime = int(h['host_in_downtime'])
+                        self.new_hosts[host_name].flapping = bool(int(h['host_is_flapping']))
+                        self.new_hosts[host_name].acknowledged = bool(int(h['host_acknowledged']))
+                        self.new_hosts[host_name].scheduled_downtime = bool(int(h['host_in_downtime']))
                         self.new_hosts[host_name].status_type = status_type
                         
                         # extra Icinga properties to solve https://github.com/HenriWahl/Nagstamon/issues/192
                         # acknowledge needs host_description and no display name
                         self.new_hosts[host_name].real_name = h['host_name']
+
+                        # Icinga only updates the attempts for soft states. When hard state is reached, a flag is set and
+                        # attemt is set to 1/x.
+                        if (status_type == 'hard'):
+                            try:
+                                maxAttempts = h['host_attempt'].split('/')[1]
+                                self.new_hosts[host_name].attempt = "{0}/{0}".format(maxAttempts)
+                            except Exception:
+                                self.new_hosts[host_name].attempt = "HARD"
        
                         # extra duration needed for calculation
                         duration = datetime.datetime.now() - datetime.datetime.fromtimestamp(int(h['host_last_state_change']))
@@ -241,7 +278,6 @@ class IcingaWeb2Server(GenericServer):
                         host_name = s['host_display_name']
 
                     # host objects contain service objects
-                    # ##if not self.new_hosts.has_key(host_name):
                     if not host_name in self.new_hosts:
                         self.new_hosts[host_name] = GenericHost()
                         self.new_hosts[host_name].name = host_name
@@ -250,13 +286,16 @@ class IcingaWeb2Server(GenericServer):
                         # acknowledge needs host_description and no display name
                         self.new_hosts[host_name].real_name = s['host_name']
 
-                    if self.use_display_name_host == False:
-                        # legacy Icinga adjustments
-                        if 'service_description' in s: service_name = s['service_description']
-                        elif 'description' in s: service_name = s['description']
-                        elif 'service' in s: service_name = s['service']
-                    else:
-                        service_name = s['service_display_name']
+                    #if self.use_display_name_host == False:
+                    #    # legacy Icinga adjustments
+                    #    if 'service_description' in s: service_name = s['service_description']
+                    #    elif 'description' in s: service_name = s['description']
+                    #    elif 'service' in s: service_name = s['service']
+                    #else:
+                    #    service_name = s['service_display_name']
+                    # regarding to https://github.com/HenriWahl/Nagstamon/issues/400 Icinga2 needs no legacy adjustments
+                    service_name = s['service_display_name']
+
 
                     # if a service does not exist create its object
                     if not service_name in self.new_hosts[host_name].services:
@@ -270,14 +309,23 @@ class IcingaWeb2Server(GenericServer):
                         self.new_hosts[host_name].services[service_name].status_information = BeautifulSoup(s['service_output'].replace('\n', ' ').strip(), 'html.parser').text
                         self.new_hosts[host_name].services[service_name].passiveonly = not(int(s['service_active_checks_enabled']))
                         self.new_hosts[host_name].services[service_name].notifications_disabled = not(int(s['service_notifications_enabled']))
-                        self.new_hosts[host_name].services[service_name].flapping = int(s['service_is_flapping'])
-                        self.new_hosts[host_name].services[service_name].acknowledged = int(s['service_acknowledged'])
-                        self.new_hosts[host_name].services[service_name].scheduled_downtime = int(s['service_in_downtime'])
+                        self.new_hosts[host_name].services[service_name].flapping = bool(int(s['service_is_flapping']))
+                        self.new_hosts[host_name].services[service_name].acknowledged = bool(int(s['service_acknowledged']))
+                        self.new_hosts[host_name].services[service_name].scheduled_downtime = bool(int(s['service_in_downtime']))
                         self.new_hosts[host_name].services[service_name].status_type = status_type
                         
                         # extra Icinga properties to solve https://github.com/HenriWahl/Nagstamon/issues/192
                         # acknowledge needs service_description and no display name
                         self.new_hosts[host_name].services[service_name].real_name = s['service_description']
+
+                        # Icinga only updates the attempts for soft states. When hard state is reached, a flag is set and
+                        # attemt is set to 1/x.
+                        if (status_type == 'hard'):
+                            try:
+                                maxAttempts = s['service_attempt'].split('/')[1]
+                                self.new_hosts[host_name].services[service_name].attempt = "{0}/{0}".format(maxAttempts)
+                            except Exception:
+                                self.new_hosts[host_name].services[service_name].attempt = "HARD"
                         
                         # extra duration needed for calculation
                         duration = datetime.datetime.now() - datetime.datetime.fromtimestamp(int(s['service_last_state_change']))
@@ -331,7 +379,32 @@ class IcingaWeb2Server(GenericServer):
         result = self.FetchURL(url, giveback='raw', cgi_data=cgi_data)
 
 
-    def _set_acknowledge(self, host, service, author, comment, sticky, notify, persistent, all_services=[]):
+    # Overwrite function from generic server to add expire_time value
+    def set_acknowledge(self, info_dict):
+        '''
+            different monitors might have different implementations of _set_acknowledge
+        '''
+        if info_dict['acknowledge_all_services'] is True:
+            all_services = info_dict['all_services']
+        else:
+            all_services = []
+
+        # Make sure expire_time is set
+        #if not info_dict['expire_time']:
+        #    info_dict['expire_time'] = None
+
+        self._set_acknowledge(info_dict['host'],
+                              info_dict['service'],
+                              info_dict['author'],
+                              info_dict['comment'],
+                              info_dict['sticky'],
+                              info_dict['notify'],
+                              info_dict['persistent'],
+                              all_services,
+                              info_dict['expire_time'])
+
+
+    def _set_acknowledge(self, host, service, author, comment, sticky, notify, persistent, all_services=[], expire_time=None):
         # First retrieve the info page for this host/service
         if service == '':
             # url = self.monitor_cgi_url + '/monitoring/host/acknowledge-problem?host=' + host
@@ -369,13 +442,16 @@ class IcingaWeb2Server(GenericServer):
         cgi_data['sticky'] = int(sticky)
         cgi_data['notify'] = int(notify)
         cgi_data['comment'] = comment
+        if expire_time:
+            cgi_data['expire'] = 1
+            cgi_data['expire_time'] = expire_time
 
         self.FetchURL(url, giveback='raw', cgi_data=cgi_data)
 
         if len(all_services) > 0:
             for s in all_services:
                 # cheap, recursive solution...
-                self._set_acknowledge(host, s, author, comment, sticky, notify, persistent, [])
+                self._set_acknowledge(host, s, author, comment, sticky, notify, persistent, [], expire_time)
 
 
     def _set_submit_check_result(self, host, service, state, comment, check_output, performance_data):
@@ -505,3 +581,55 @@ class IcingaWeb2Server(GenericServer):
             self.Debug(server=self.get_name(), host=host, service=service,
                        debug='Open host/service monitor web page {0}'.format(url))
         webbrowser_open(url)
+
+    def GetHost(self, host):
+        '''
+            find out ip or hostname of given host to access hosts/devices which do not appear in DNS but
+            have their ip saved in Icinga
+        '''
+        # Host is the display name as in the GUI
+        # but we need the FQDN not the display name
+        host = self.hosts[host].real_name
+
+        # the fasted method is taking hostname as used in monitor
+        if conf.connect_by_host is True or host == '':
+            return Result(result=host)
+
+        # initialize ip string
+        ip = ''
+        address = ''
+
+        # glue nagios cgi url and hostinfo
+        cgiurl_host = self.monitor_cgi_url + '/monitoring/list/hosts?host={0}&addColumns=host_address&format=json'.format(host)
+
+        # get host info
+        hostobj = self.FetchURL(cgiurl_host, giveback='raw')
+        jsonhost = hostobj.result
+
+        try:
+            # take ip from json output
+            result = json.loads(jsonhost)[0]
+            ip = result["host_address"]
+
+            # print IP in debug mode
+            if conf.debug_mode is True:
+                self.Debug(server=self.get_name(), host=host, debug='IP of %s:' % (host) + ' ' + ip)
+
+            # when connection by DNS is not configured do it by IP
+            if conf.connect_by_dns is True:
+                # try to get DNS name for ip, if not available use ip
+                try:
+                    address = socket.gethostbyaddr(ip)[0]
+                except socket.error:
+                    address = ip
+            else:
+                address = ip
+        except Exception:
+            result, error = self.Error(sys.exc_info())
+            return Result(result=result, error=error)
+
+        # do some cleanup
+        del hostobj
+
+        # give back host or ip
+        return Result(result=address)
